// Fill out your copyright notice in the Description page of Project Settings.


#include "Ocean.h"

#include "DrawDebugHelpers.h"

// Sets default values
AOcean::AOcean()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	XSize = YSize = 10.0f;
	
	OceanMesh = CreateDefaultSubobject<UProceduralMeshComponent>(TEXT("OceanMesh"));
	OceanMesh->SetupAttachment(RootComponent);

	OceanMesh->SetCollisionEnabled(ECollisionEnabled::NoCollision);
}

float AOcean::GetOceanHeight()
{
	/**
	 * For now assume a completely flat static ocean.
	 * Once we get into procedural ocean and wave generation we will want to be able to pass in a world X and Y position
	 * and have this function return the height of the ocean at that location 
	 */ 
	return GetActorLocation().Z;
}

// Called when the game starts or when spawned
void AOcean::BeginPlay()
{
	Super::BeginPlay();

	GenerateMesh();
}

// Called every frame
void AOcean::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

	for (auto Vertex : Vertices)
	{
		DrawDebugSphere(GetWorld(), Vertex, 2.0f, 6, FColor::Red, false, 0.0f, -1, 1.0);	
	}
}

void AOcean::GenerateMesh()
{
	Vertices = TArray<FVector>();
	Vertices.SetNum((XSize + 1) * (YSize + 1));
	UV0 = TArray<FVector2D>();
	UV0.SetNum(Vertices.Num());
	Tangents = TArray<FProcMeshTangent>();
	Tangents.SetNum(Vertices.Num());
	FProcMeshTangent Tangent;
	for(uint32 i = 0, y = 0; y <= YSize; y++)
	{
		for(uint32 x = 0; x <= XSize; x++, i++)
		{
			Vertices[i] = FVector(x * 100.0f, y * 100.0f, 0.0f);
			UV0[i] = FVector2D((float)x/XSize, (float)y/YSize);
			Tangents[i] = Tangent;
		}
	}

	Triangles = TArray<int32>();
	Triangles.SetNum(XSize * YSize * 6);
	for(uint32 ti = 0, vi = 0, y = 0; y < YSize; y++, vi++)
	{
		for(uint32 x = 0; x < XSize; x++, ti += 6, vi++)
		{
			Triangles[ti] = vi;
			Triangles[ti + 3] = Triangles[ti + 2] = vi + 1;
			Triangles[ti + 4] = Triangles[ti + 1] = vi + XSize + 1;
			Triangles[ti + 5] = vi + XSize + 2;
		}
	}
	
	OceanMesh->CreateMeshSection(1, Vertices, Triangles, Normals, UV0, VertexColors, Tangents, false);
}

